'use strict';

const path = require('path')
const cp = require('child_process');
const Packages = require('@gl-cli-dev/packages')
const log = require('@gl-cli-dev/log')

const SETTINGS = {
  init: '@gl-cli-dev/init'
}
const CATCH_DIR = 'dependencies/'

async function exec() {
  let storeDir, pkg;
  let targetPath = process.env.CLI_TARGET_PATH;
  const homePath = process.env.CLI_HOME_PATH;
  log.verbose('targetPath', targetPath)
  log.verbose('homePath', homePath)
  const cmdObj = arguments[arguments.length - 1];
  const cmdName = cmdObj.name();
  const packageName = SETTINGS[cmdName];
  const packageVersion = 'latest'; //最新版本 latest

  //不指定调试文件，直接使用缓存 @gl-cli-dev/init
  if (!targetPath) {
    //生成缓存路径
    targetPath = path.resolve(homePath, CATCH_DIR);
    storeDir = path.resolve(targetPath, 'node_modules');
    log.verbose('targetPath', targetPath)
    log.verbose('storeDir', storeDir)
    pkg = new Packages({
      targetPath,
      storeDir,
      packageName,
      packageVersion
    });
    if (await pkg.exists()) {
      //更新package
      await pkg.update()
    } else {
      //console.log('install.....')
      //安装package
      await pkg.install()
    }
  } else {
    pkg = new Packages({
      targetPath,
      packageName,
      packageVersion
    })
    log.verbose('pkg', pkg)
  }

  //开启新的线程执行init操作
  const rootFile = pkg.getRootFilePath()
  if (rootFile) {
    try {
      let args = Array.from(arguments);
      const cmd = args[args.length - 1]
      const o = Object.create(null);
      Object.keys(cmd).forEach(key => {
        if (cmd.hasOwnProperty(key) && !key.startsWith('_') && key !== 'parent') {
          o[key] = cmd[key]
        }
      })
      args = args.slice(0, args.length - 1);
      args[args.length - 1] = o;
      const code = `require(${JSON.stringify(rootFile)}).call(null, ${JSON.stringify(args)})`;
      const child = spawn('node', ['-e', code], {
        cwd: process.cwd(),
        stdio: 'inherit'
      })
      child.on('error', e => {
        log.error(e.message)
        process.exit(1)
      })
      child.on('exit', e => {
        log.verbose('命令执行成功：' + e)
        process.exit(e)
      })

    } catch (e) {
      log.error(e.message)
    }
  }
}

//兼容windows
function spawn(command, args, options) {
  const win32 = process.platform === 'win32';
  const cmd = win32 ? 'cmd' : command;
  const cmdArgs = win32 ? ['/c'].concat(command, args) : args;
  return cp.spawn(cmd, cmdArgs, options || {})
}

module.exports = exec;
